#!/usr/bin/env perl

# usage: cat XXX.xml | perl conv_format_juman_knp.pl -id_base ntcir10_rite2_bc-dev

# JUMAN/KNP解析結果をうめこむプログラム

use strict;
use utf8;
binmode(STDIN, ':encoding(utf8)');
binmode(STDOUT, ':encoding(utf8)');
binmode(STDERR, ':encoding(utf8)');
use Encode;
use KNP;
use Getopt::Long;
use XML::LibXML;
use AddKNPResult;
use Unicode::Japanese;

my (%opt);
GetOptions(\%opt, 'id_base=s', 'id=s', 'bnst', 'debug', 'knp_command=s', 'knp_rcfile=s', 'juman_command=s', 'juman_rcfile=s', 'simple');
# id_base: Sentenceタグの属性のidのbaseを指定
#  例えば、-id_base ntcir10_rite2_bc-dev と指定すると、1文目の<t1>の<Sentence>のidは ntcir10_rite2_bc-dev_1_t1となる
# id: 指定されたidのみを処理する
#   テスト時 
# bnst: Chunkの単位を文節にする(このオプションを指定しないと、Chunkの単位は基本句になる)
# knp_command, knp_rcfile, juman_command, juman_rcfile: パスに通っていないJUMAN/KNPを利用する場合に指定
# simple: 必要最小限のタグを出力
 
my $writer = new XML::LibXML::Document->new('1.0', 'utf-8');

my $knp_option = { -Timeout => 0 };
$knp_option->{'-Command'} = $opt{knp_command} if $opt{knp_command};
$knp_option->{'-Rcfile'} = $opt{knp_rcfile} if $opt{knp_rcfile};
$knp_option->{'-JumanCommand'} = $opt{juman_command} if $opt{juman_command};
$knp_option->{'-JumanRcfile'} = $opt{juman_rcfile} if $opt{juman_rcfile};

my $knp = new KNP($knp_option);

my $addknpresult = new AddKNPResult(\%opt);

my ($buf);
while (<STDIN>) {
    $buf .= $_;
}
my $parser = new XML::LibXML;
my $doc = $parser->parse_string($buf);

my $option;
$option->{simple} = 1 if $opt{simple};

for my $pair ($doc->getElementsByTagName('pair')) {
    my $id = $pair->getAttribute('id');
    next if $opt{id} && $opt{id} ne $id;
    for my $child ($pair->getChildNodes) {
	my $nodename = $child->nodeName;
	if ($nodename eq 't1' || $nodename eq 't2') {
	    my $text = $child->getFirstChild->data;

	    my $sentence_node = $writer->createElement('Sentence');
	    $sentence_node->setAttribute('text', $text);

	    # role
	    my $role = $nodename eq 't1' ? 'text' : 'hypothesis';
	    $sentence_node->setAttribute('role', $role);

	    # id
	    my $id_string = $opt{id_base} . '_' . $id . '_' . $nodename;
	    $sentence_node->setAttribute('id', $id_string);

	    $text = &preprocess_text($text);
	    print STDERR "$id $nodename $text\n" if $opt{debug};
	    my $result = $knp->parse($text);

	    $addknpresult->Annotation2XMLforKNP($writer, $result, $sentence_node, $option);
	    $child->appendChild($sentence_node);
	}
    }
}

my $string = $doc->toString(1); # 1 means indenting
print utf8::is_utf8($string) ? $string : decode('utf-8', $string);

sub preprocess_text {
    my ($string) = @_;

    $string =~ s/ //g;
    $string = Unicode::Japanese->new($string)->h2z->getu;

    return $string;
}
